clear all
cls
version 14
est clear
graph drop _all
scalar drop _all
matrix drop _all
set type float
set seed 33333
set more off
set logtype text
capture log close
cap program drop bunchfilter
use bunching.dta



*the friction error only exists plus and minus deltap,deltam  around the kink
scalar deltap = ln(1.1)
scalar deltam = -ln(0.9)

*use the tax schedule for one child given on page 190 of Saez (2010);
scalar s0 = log(1-0.3)
scalar s1 = log(1+0.1)


*kink location
scalar kink =  round(ln(8),0.0001)



*randomly select 10K observations
sample 10000, count



**********default run
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') pctobs(30) polorder(7)




*********test option generate
*error: missing option
cap noisily  bunchfilter yfric , kink(`=kink') deltam(`=deltam') deltap(`=deltap') 

*error: variable already exists 
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')

*abbreviation
cap drop yfilter
cap noisily  bunchfilter yfric , gen(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  




*********test option kink

*error missing
cap drop yfilter
cap noisily  bunchfilter yfric , gen(yfilter)  deltam(`=deltam') deltap(`=deltap')

*abbreviation
cap drop yfilter
cap noisily  bunchfilter yfric , gen(yfilter) k(`=kink') deltam(`=deltam') deltap(`=deltap')  


*look for extreme values to move the kink to
sum y, d

*error: no obs on the left
cap drop yfilter
cap noisily  bunchfilter yfric , gen(yfilter) kink(-0.92) deltam(`=deltam') deltap(`=deltap')


*error: no obs on the rigth
cap drop yfilter
cap noisily  bunchfilter yfric , gen(yfilter) kink(5.93) deltam(`=deltam') deltap(`=deltap')




*error: kink not extreme but not true either
cap drop yfilter
cap noisily  bunchfilter yfric , gen(yfilter) kink(0) deltam(`=deltam') deltap(`=deltap')



*********test option deltas
*error missing
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltap(`=deltap')  
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(`=deltam')  

*error non str positive
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(-0.326) deltap(`=deltap')  
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(`=deltam')  deltap(-0.326)  
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(0)  deltap(0.1)  


*error too big
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(10) deltap(10)  

*deltas are zero
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(0) deltap(0)  



*assymetric deltas
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(`=deltam') deltap(0.05)  

cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(0.05) deltap(`=deltap')  

cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink')  deltam(.6) deltap(`=deltap')  


*********test option weights
*error abbreviation
cap drop yfilter
cap noisily  bunchfilter yfric [fweights=w], generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  

*abbreviation
cap drop yfilter
cap noisily  bunchfilter yfric [fw=w], generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  

*without weights
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') 
return list

*with weights
cap drop yfilter
cap noisily  bunchfilter yfric [fw=w], generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') 
return list

*with different weights
cap noisily drop yfilter
gen w2=_n^4
cap noisily  bunchfilter yfric [fw=w2], generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  
return list



*********test sample selection
cap noisily drop yfilter
cap noisily bunchfilter y [fw=w] if (y<`=kink'-`=deltam')|(y>`=kink'+`=deltap'), gen(yfilter) k(`=kink') deltam(`=deltam') deltap(`=deltap')  
return list


cap noisily drop yfilter
cap noisily bunchfilter y [fw=w] if (y<`=kink'-.1)|(y>`=kink'+.1), gen(yfilter) k(`=kink') deltam(`=deltam') deltap(`=deltap')    
return list

preserve
drop if (y>=`=kink'-`=deltap')&(y<=`=kink'+`=deltam')
cap noisily drop yfilter
cap noisily bunchfilter y [fw=w] , gen(yfilter) k(`=kink') deltam(`=deltam') deltap(`=deltap')  
return list
restore


preserve
drop if (y>=`=kink'-.1)&(y<=`=kink'+.1)
cap noisily drop yfilter
cap noisily bunchfilter y [fw=w] , gen(yfilter) k(`=kink') deltam(`=deltam') deltap(`=deltap')  
return list
restore


*********test option nopic
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') nopic
return list



*********test option binwidth
*error: not str positive
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') binwidth(-.1) 
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') binwidth(0) 



*abbreviation
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') binw(0.08) 
return list


*difference in return() with option nopic
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') binw(0.08) nopic
return list




*********test option pctobs
*error: invalid numbers
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pctobs(-1)
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pctobs(0)
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pctobs(50.5)
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pct(100)



*abbreviation
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pct(50)


*small value
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pct(10)

*large value
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap')  pct(99)



*********test option polorder
*error: invalid numbers
cap drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') polorder(0)
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') polorder(8)
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') polorder(4.73)


*abbreviation small value
cap noisily drop yfilter
cap noisily  bunchfilter yfric , generate(yfilter) kink(`=kink') deltam(`=deltam') deltap(`=deltap') pol(2)
return list



