clear all
cls
version 14
est clear
graph drop _all
scalar drop _all
matrix drop _all
set type float
set seed 33333
set more off
set logtype text
capture log close
cap program drop bunchtobit
cap program drop bunchtobit_tloglike
use bunching.dta

*the friction error only exists plus and minus deltap,deltam  around the kink
scalar deltap = ln(1.1)
scalar deltam = -ln(0.9)

*use the tax schedule for one child given on page 190 of Saez (2010);
scalar s0 = log(1-(-0.3))
scalar s1 = log(1-(+0.1))


*kink location
scalar kink =  round(ln(8),0.0001)

**randomly select a subsample 
*sample 10000, count

***default run 
cap noisily bunchtobit y x1 x2 [fw=w], kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) savingtobit(bmstobit, replace) verbose
return list
preserve
use bmstobit, replace
list
restore


***option binwidth


cap noisily bunchtobit y x1 x2 [fw=w], kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) grid(30 90) binw(0.08)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) grid(30 90) binwidth(0.15)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) grid(30 90) binwidth(0)


***kink option and Bhat<=1/n
cap noisily bunchtobit y x1 x2, k(`=kink') s0(`=s0') s1(`=s1') numiter(50) nopic grid(30 90) 



cap noisily  bunchtobit y x1 x2, s0(`=s0') s1(`=s1') numiter(50)  


*kink to a point without bunching mass
cap drop bunch
gen bunch = (y==1)
sum bunch
drop bunch
cap noisily  bunchtobit y x1 x2, kink(1) s0(`=s0') s1(`=s1') numiter(50) grid(90 100)



*test convergence of initial tobits
*make sure there are 2 obs equal the min and max values in the sample (so it passes Bhat>0 test)
sort y
replace y=8.55 if _n>=_N-1 
replace y=-2.72 if _n<=2

cap noisily  bunchtobit y x1 x2, kink(8.55) s0(`=s0') s1(`=s1') numiter(50) grid(90)


cap noisily  bunchtobit y x1 x2, kink(-2.72) s0(`=s0') s1(`=s1') numiter(50) grid(90)






***s0 s1 options
cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=log(1-.5)') numiter(50)
 
cap noisily bunchtobit y x1 x2, kink(`=kink') s1(`=log(1-.5)') numiter(50) nopic grid(30 90)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=log(1-.5)') numiter(50) nopic grid(30 90)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=log(1-.1)') numiter(50) grid(30 90)
 




***varlist
cap noisily bunchtobit , kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) grid(30 90)
cap noisily bunchtobit y x*, kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) grid(30 90) verbose nopic



***numiter
cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') n(50) nopic grid(30 90)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') n(-1) nopic grid(30 90)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(-1) nopic grid(30 90)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(1.56) nopic grid(30 90)

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(1) nopic grid(30 90)




***grid
cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(80(1)90) n(10) 

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(70(.1)80) n(10) 


cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(-10(10)50) n(10) 

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(80 90) n(10) 

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(80 85 to 90) n(10) 

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(85/90) n(10) 

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(90[-10]70) n(10) 

cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') grid(90 70 50) n(10) 

 



****option of weights
*error: fweightS
cap noisily bunchtobit y x1 x2 [fweights = w], k(`=kink') s0(`=s0') s1(`=s1') 

cap noisily bunchtobit y x1 x2 [fweight = w], k(`=kink') s0(`=s0') s1(`=s1') 


cap noisily bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1')

***no weights
cap noisily bunchtobit y x1 x2, k(`=kink') s0(`=s0') s1(`=s1')




*****sample selection options
cap noisily bunchtobit y x1 x2 in 4000/67000, k(`=kink')  s0(`=s0') s1(`=s1') grid(30 90) verbose savingtobit(bmstobit, replace)
preserve
use bmstobit, replace
list
restore


cap noisily bunchtobit y x1 x2 in 8000/25000, k(`=kink')  s0(`=s0') s1(`=s1') grid(30 90) verbose savingtobit(bmstobit, replace)
preserve
use bmstobit, replace
list
restore



cap noisily bunchtobit y x1 x2 if y>-.77 & y<4.3 [fw = w], k(`=kink')  s0(`=s0') s1(`=s1') grid(30 90) savingtobit(bmstobit, replace)
preserve
use bmstobit, replace
list
restore



***saving option
cap noisily  bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1') n(50) nopic savingtobit(bmstobit) grid(80 90)
cap noisily  bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1') n(50) nopic savingtobit("bmstobit") grid(80 90)
cap noisily  bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1') n(50) nopic savingtobit("bmstobit", replace) grid(80 90)


cap noisily bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1') n(50) nopic savingtobit(bmstobit.dta) grid(80 90)
cap noisily bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1') n(50) nopic savingtobit("bmstobit.dta") grid(80 90)
cap noisily bunchtobit y x1 x2 [fw = w], k(`=kink') s0(`=s0') s1(`=s1') n(50) nopic savingtobit(bmstobit.dta, replace) grid(80 90)





******collinearity tests
**create grid 10(10)90 + 100
scalar ngrid_p = 9  
scalar max_p = 90  
scalar min_p = 10  
scalar bin_p = (max_p-min_p)/(ngrid_p-1)
matrix perc_obs = J(ngrid_p,1,0)
matrix grid_h = J(ngrid_p,1,0)
gen temp = abs(y -`=kink')
forval i=1/`=ngrid_p' {
	scalar m = ngrid_p - (`i'-1)
	matrix perc_obs[m,1]=round(min_p + bin_p*(`i'-1),1)	
	_pctile temp [fweight= w], percentiles(`=perc_obs[m,1]')
	matrix grid_h[m,1]=`r(r1)'
}
qui sum temp [fweight= w]
scalar large_grid_h = 10 * r(max)
drop temp
matrix perc_obs = 100\perc_obs
matrix grid_h = (large_grid_h)\grid_h
scalar ngrid_p = ngrid_p + 1

***generate redundant covariates to test the parts of the code that drop collinear covariates
cap drop x3
cap drop x4 
cap drop x5
cap drop x6
cap drop x7 

** x3 should be dropped in the pre-estimates stage: dropped from both 2-sided tobits
gen x3 = x1+x2 

** x4 should be dropped in trunc window #3
scalar h = grid_h[3,1]
gen x4 = rnormal()
replace x4 = x1+x2 if y<kink+h & y>kink-h

** x5 should be dropped in trunc window #7
scalar h = grid_h[7,1]
gen x5 = rnormal()
replace x5 = x1+x2 if y<kink+h & y>kink-h


*scalar drop _all
matrix drop _all


cap noisily bunchtobit y x1 x2 x3 x4 x5 [fw = w], kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) savingtobit(bmstobit, replace) verbose
preserve
use bmstobit, replace
list
restore

drop x3 x4 x5 


*sampledata.dta contains a sample dataset where one of the two covariates has an estimate but no S.E. (collinearity?) in just one side of the 2-sided tobit
*the previous version of bunchtobit would drop this covariate before getting to the mid-censored tobit
*this version of bunchtobit does not drop it
preserve
use sampledata, replace
sample 50000, count
cap noisily bunchtobit y x1 x2, kink(`=round(ln(8.580),.0001)') grid(90) s0(`=s0') s1(`=s1') verbose
restore






*** consistency without weights



*50K observations
clear
use bunching
sample 50000, count
cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(50) 


*100K observations
clear
use bunching
cap noisily bunchtobit y x1 x2, kink(`=kink') s0(`=s0') s1(`=s1') numiter(50)




